/*
 *  This file is part of the XForms library package.
 *
 *  XForms is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU Lesser General Public License as
 *  published by the Free Software Foundation; either version 2.1, or
 *  (at your option) any later version.
 *
 *  XForms is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with XForms.  If not, see <http://www.gnu.org/licenses/>.
 */
/* CHAR_SET_DEPENDEND */

/**
 * \file char_dep.c
 *
 *  This file is part of the XForms library package.
 *  Copyright (c) 1996-2002  T.C. Zhao
 *  All rights reserved.
 *.
 *  puting char set depencies into one file, to bring ISO8859-1 versions
 *  and utf8-versions closer together.
 *
 */

#include "include/forms.h"

#if FL_ENABLE_XFT
    #include "utf8.h"
#endif


/***************************************
 * Returns the number of bytes in an UTF-8 character
 * (or -1 if it's not a valid UYF8 character)
 ***************************************/

int
char_get_char_bytes( int c )

{
#if FL_ENABLE_XFT
    return utf8_get_char_bytes ( c );
#else
    return 1;
#endif
}


/***************************************
 * Returns the number of bytes in the UTF-8 character
 * pointed to by a char pointer (or -1 if it's not a
 * valid UTF-8 character)
 *
 * in ISO8859-1 it always returns 1
 ***************************************/

int 
char_get_byte_count ( const char * str FL_UNUSED_ARG)

{
#if FL_ENABLE_XFT
    return utf8_get_byte_count( str);
#else
    return 1;
#endif
}


/***************************************
 * Returns the number of bytes in the UTF-8 character
 * before the string pointed to (or -1 if it's no a
 * valid UTF-8 character)
 ***************************************/

int
char_get_prev_byte_count( const char * str )

{
#if FL_ENABLE_XFT
    return utf8_get_prev_byte_count( str );
#else
    const unsigned char * p = ( const unsigned char * ) str;

    if ( *--p != 0x0 )
        return 1;
    return -1;
#endif
}


/***************************************
 * Returns a pointer to the next character after the start of 
 * an UTF-8 string or ISO8859-1  string
 ***************************************/

char *
char_next_char_pos ( char * str  )

{
#if FL_ENABLE_XFT
    return str + utf8_get_byte_count( str );
#else
    return str + 1;
#endif
}


/***************************************
 * Returns a pointer to the character before the start of an UTF-8 or 
 * iso 8859-1 string
 * (this, of course, assumes that the pointer passed to the function
 * points into a string).
 ***************************************/

char *
char_prev_char_pos ( char * str )

{
#if FL_ENABLE_XFT
    return str - utf8_get_prev_byte_count( str );
#else
    return str - 1;
#endif
}


/*
 * Local variables:
 * tab-width: 4
 * indent-tabs-mode: nil
 * End:
 */
